@@ -2,15 +2,13 @@ import argparse
 from . import cfg
 from .commands import (create, update, delete, cancel_update, continue_update,
                        info, parameters, resolve, show_table, log, dash,
-                       ssm_setup, ssm_put, ssm_show)
+                       ssm_setup, ssm_put, ssm_show, r53, replicate)
 
 
-def set_create_parser(subparser, parents=[]):
+def get_create_parser(subparser, parents=[]):
     parser = subparser.add_parser('create',
                                   parents=parents,
                                   help='Create Stack')
-    parser.set_defaults(func=create)
-
     parser.add_argument('--Env',
                         help='Environment to use',
                         type=str, required=True)
@@ -21,20 +19,13 @@ def set_create_parser(subparser, parents=[]):
                         help='App Version',
                         type=str, default='')
 
+    return parser
 
-def set_update_parser(subparser, parents=[]):
+
+def get_update_parser(subparser, parents=[]):
     parser = subparser.add_parser('update',
                                   parents=parents,
                                   help='Update Stack')
-    parser.set_defaults(func=update)
-    # used by dashboard update
-    parser.set_defaults(
-        statistic='Average',
-        statisticresponse='p95',
-        debug=False,
-        silent=True,
-        vertical=False)
-
     parser.add_argument('-P', '--policy',
                         help='Policy during Stack Update',
                         type=str, choices=[
@@ -59,6 +50,8 @@ def set_update_parser(subparser, parents=[]):
                         help='Do not show extra details in changeset',
                         action='store_true')
 
+    return parser
+
 
 def set_dash_parser(subparser, parents=[]):
     parser = subparser.add_parser('dash',
@@ -93,17 +86,19 @@ def set_dash_parser(subparser, parents=[]):
     )
 
 
-def set_show_parser(subparser, parents=[]):
+def get_show_parser(subparser, parents=[]):
     parser = subparser.add_parser('show',
                                   parents=parents,
                                   help='Show Stacks table')
-    parser.set_defaults(func=show_table)
+
     parser.add_argument('-F', '--fields', nargs='+',
                         type=str, default=cfg.SHOW_TABLE_FIELDS)
     parser.add_argument('-O', '--output',
                         type=str, default='text',
                         choices=['text', 'html', 'bare'])
 
+    return parser
+
 
 def set_ssm_parser(subparser, parents=[]):
     parser = subparser.add_parser(
@@ -111,28 +106,97 @@ def set_ssm_parser(subparser, parents=[]):
         parents=[],
         help='SSM Parameters override for Stack Replicator')
 
-    regions_parser = argparse.ArgumentParser(add_help=False)
-    regions_parser.add_argument('-R', '--regions',
-                                help='Regions', type=str,
-                                required=True, default=[], nargs='+')
-
     ssm_parser = parser.add_subparsers(title='SSM Command',
                                        required=True, dest='command_ssm')
-
+    # setup
     setup_parser = ssm_parser.add_parser(
         'setup', help='Setup Regions',
-        parents=parents + [regions_parser])
-    setup_parser.set_defaults(func=ssm_setup)
+        parents=parents)
+    setup_parser.set_defaults(func=ssm_setup, no_stacks=True)
 
+    setup_parser.add_argument('-R', '--regions',
+                              help='Regions', type=str,
+                              required=True, default=[], nargs='+')
+    # put
     put_parser = ssm_parser.add_parser(
         'put', help='Put Parameters',
-        parents=parents + [regions_parser])
+        parents=parents)
     put_parser.set_defaults(func=ssm_put)
 
+    put_parser.add_argument('-R', '--regions',
+                            help='Regions', type=str,
+                            default=[], nargs='+')
+    # show
     show_parser = ssm_parser.add_parser(
         'show', help='Show Regions Distribution',
         parents=parents)
-    show_parser.set_defaults(func=ssm_show)
+    show_parser.set_defaults(func=ssm_show, all_stacks=True)
+
+
+def set_replicate_parser(subparser, parents=[]):
+    parser = subparser.add_parser(
+        'replicate',
+        parents=[],
+        help='Replicate in Regions configured by SSM')
+    parser.set_defaults(func=replicate)
+
+    replicate_parser = parser.add_subparsers(title='Replicate Command',
+                                             required=True,
+                                             dest='command_replicate')
+
+    regions_parser = argparse.ArgumentParser(add_help=False)
+    regions_parser.add_argument('-R', '--regions',
+                                help='Regions', type=str,
+                                default=[], nargs='+')
+    regions_parser.add_argument('--no_replicate_current',
+                                help='No replication in current Region',
+                                action='store_true')
+
+    # replicate create
+    parser_create = get_create_parser(
+        replicate_parser, parents + [
+            get_template_parser(),
+            get_stack_single_parser(),
+            get_create_update_parser(),
+            regions_parser])
+
+    # replicate update
+    parser_update = get_update_parser(
+        replicate_parser, parents + [
+            get_template_parser(required=False),
+            get_stack_selection_parser(),
+            get_create_update_parser(),
+            regions_parser])
+
+    # replicate delete
+    parser_delete = replicate_parser.add_parser(
+        'delete',
+        parents=parents + [
+            get_stack_single_parser(),
+            regions_parser],
+        help='Delete Stack (WARNING)')
+
+    # replicate show
+    parser_show = get_show_parser(
+        replicate_parser, [
+            get_stack_selection_parser(),
+            regions_parser])
+
+
+def set_r53_parser(subparser, parents=[]):
+    parser = subparser.add_parser(
+        'r53', parents=parents,
+        help='Create RecordSet Aliases looking at stack R53 resources')
+    parser.set_defaults(func=r53)
+
+    parser.add_argument('--dryrun', help='Show changes and exit',
+                        action="store_true")
+    parser.add_argument('--noorigin', help='Do not create Origin Record',
+                        action="store_true")
+    parser.add_argument(
+        '--suffix',
+        help='Suffix to add to RecordSet Name, prepended with "-"',
+        default='', type=str)
 
 
 def get_template_parser(required=True):
@@ -168,6 +232,31 @@ def get_stack_selection_parser():
     return parser
 
 
+def get_stack_single_parser():
+    parser = argparse.ArgumentParser(add_help=False)
+
+    parser.add_argument(
+        '-s', '--stack', nargs=1,
+        help='Stack Names space separated',
+        required=True, type=str, default=[])
+
+    return parser
+
+
+def get_create_update_parser():
+    parser = argparse.ArgumentParser(add_help=False)
+
+    parser.add_argument('--topics', nargs='+',
+                        help='SNS Topics Arn for notification',
+                        type=str, default=[])
+    parser.add_argument('-M', '--max_retry_ecs_service_running_count',
+                        help='Max retry numbers when updating ECS '
+                             'service and runningCount is stuck to zero',
+                        type=int, default=0)
+
+    return parser
+
+
 # parse main argumets
 def get_parser():
     parser = argparse.ArgumentParser(
@@ -214,22 +303,10 @@ def get_parser():
     stack_selection_parser = get_stack_selection_parser()
 
     # stack single parser
-    stack_single_parser = argparse.ArgumentParser(add_help=False)
-    stack_single_parser.add_argument(
-        '-s', '--stack', nargs=1,
-        help='Stack Names space separated',
-        required=True, type=str, default=[])
-    # update create parser
-    updcrt_parser = argparse.ArgumentParser(add_help=False)
+    stack_single_parser = get_stack_single_parser()
 
-    updcrt_parser.add_argument('--topics', nargs='+',
-                               help='SNS Topics Arn for notification',
-                               type=str, default=[])
-    updcrt_parser.add_argument('-M', '--max_retry_ecs_service_running_count',
-                               help='Max retry numbers when updating ECS '
-                                    'service and runningCount is stuck to '
-                                    'zero',
-                               type=int, default=0)
+    # create update parser
+    create_update_parser = get_create_update_parser()
 
     # command subparser
     command_subparser = parser.add_subparsers(
@@ -238,22 +315,24 @@ def get_parser():
         dest='command')
 
     # create parser
-    set_create_parser(
+    parser_create = get_create_parser(
         command_subparser, [
             action_parser,
             template_parser_create,
             stack_single_parser,
-            updcrt_parser,
+            create_update_parser,
         ])
+    parser_create.set_defaults(func=create)
 
     # update parser
-    set_update_parser(
+    parser_update = get_update_parser(
         command_subparser, [
             action_parser,
             template_parser_update,
             stack_selection_parser,
-            updcrt_parser,
+            create_update_parser,
         ])
+    parser_update.set_defaults(func=update)
 
     # delete parser
     parser_delete = command_subparser.add_parser(
@@ -282,7 +361,7 @@ def get_parser():
         help='Continue Update RollBack')
     parser_continue.set_defaults(func=continue_update)
     parser_continue.add_argument(
-        '--resources_to_skip', '-R',
+        '--resources_to_skip',
         help='Resource to Skip',
         default=[], nargs='+')
 
@@ -328,13 +407,26 @@ def get_parser():
         ])
 
     # show parser
-    set_show_parser(
+    parser_show = get_show_parser(
+        command_subparser, [
+            stack_selection_parser,
+        ])
+    parser_show.set_defaults(func=show_table, all_stacks=True)
+
+    # ssm parser
+    set_ssm_parser(
         command_subparser, [
             stack_selection_parser,
         ])
 
-    # ssm parser
-    set_ssm_parser(
+    # replicate parser
+    set_replicate_parser(
+        command_subparser, [
+            action_parser,
+        ])
+
+    # r53 parser
+    set_r53_parser(
         command_subparser, [
             stack_selection_parser,
         ])
@@ -349,4 +441,15 @@ def set_cfg(argv):
     for n, v in vars(args[0]).items():
         setattr(cfg, n, v)
 
+    # trick for showing ALL Stacks
+    # if nor stack nor role nor type are specified.
+    if (cfg.all_stacks
+            and not (cfg.stack or cfg.role or cfg.type)):
+        cfg.type = ['ALL']
+
+    if (not cfg.no_stacks
+            and not (cfg.stack or cfg.role or cfg.type)):
+        parser.print_help()
+        exit(0)
+
     cfg.stack_args = args[1]
